@@ -5,6 +5,7 @@
 * Changes:
 *   05/21/03   DGP   Original.
 *   12/08/04   DGP   Started adding MAP pseudo ops.
+*   02/02/05   DGP   Added some missing opcodes, THREE, SEVEN & ZERO.
 *	
 ***********************************************************************/
 
@@ -38,14 +39,17 @@ OpCode optable[NUMOPS] =
    { "BNT",    BNT_T,    0,     TYPE_P    },
    { "BOOL",   BOOL_T,   0,     TYPE_P    },
    { "BRA",    07000,    0,     TYPE_A    },
+   { "BRN",    03000,    0,     TYPE_A    },
    { "BSF",    04764,    0,     TYPE_E    },
    { "BSR",    00764,    0,     TYPE_E    },
    { "BSS",    BSS_T,    0,     TYPE_P    },
    { "BTT",    00760,    0,     TYPE_E    },
+   { "CAD",    04700,    0,     TYPE_B    },
    { "CAL",    04500,    0,     TYPE_B    },
    { "CALL",   CALL_T,   0,     TYPE_P    },
    { "CAQ",    04114,    0,     TYPE_C    },
    { "CAS",    00340,    0,     TYPE_B    },
+   { "CFF",    00760,    00030, TYPE_E    },
    { "CHS",    00760,    00002, TYPE_E    },
    { "CLA",    00500,    0,     TYPE_B    },
    { "CLM",    00760,    00000, TYPE_E    },
@@ -54,6 +58,7 @@ OpCode optable[NUMOPS] =
    { "COMMON", COMMON_T, 0,     TYPE_P    },
    { "CONTRL", CONTRL_T, 0,     TYPE_P    },
    { "COUNT",  COUNT_T,  0,     TYPE_P    },
+   { "CPY",    00700,    0,     TYPE_B    },
    { "CPYD",   05000,    0,     TYPE_CHAN },
    { "CPYP",   04000,    0,     TYPE_CHAN },
    { "CRQ",    04154,    0,     TYPE_C    },
@@ -128,6 +133,7 @@ OpCode optable[NUMOPS] =
    { "FUL",    FUL_T,    0,     TYPE_P    },
    { "FVE",    05000,    0,     TYPE_A    },
    { "GOTO",   GOTO_T,   0,     TYPE_P    },
+   { "HEAD",   HEAD_T,   0,     TYPE_P    },
    { "HPR",    00420,    0,     TYPE_B    },
    { "HTR",    00000,    0,     TYPE_B    },
    { "ICC",    07000,    02,    TYPE_CHAN },
@@ -172,6 +178,7 @@ OpCode optable[NUMOPS] =
    { "LCHF",   04546,    0,     TYPE_B    },
    { "LCHG",   00547,    0,     TYPE_B    },
    { "LCHH",   04547,    0,     TYPE_B    },
+   { "LDA",    00460,    0,     TYPE_B    },
    { "LDC",    04535,    0,     TYPE_B    },
    { "LDI",    00441,    0,     TYPE_B    },
    { "LDIR",   LDIR_T,   0,     TYPE_P    },
@@ -207,6 +214,7 @@ OpCode optable[NUMOPS] =
    { "MZE",    04000,    0,     TYPE_A    },
    { "NOCRS",  NOCRS_T,  0,     TYPE_P    },
    { "NOP",    00761,    0,     TYPE_B    },
+   { "NTR",    01000,    0,     TYPE_A    },
    { "NULL",   NULL_T,   0,     TYPE_P    },
    { "NZT",    04520,    0,     TYPE_B    },
    { "OAI",    00043,    0,     TYPE_B    },
@@ -256,7 +264,9 @@ OpCode optable[NUMOPS] =
    { "RCHH",   04543,    0,     TYPE_B    },
    { "RCT",    00760,    00014, TYPE_E    },
    { "RDC",    00760,    00352, TYPE_E    },
+   { "RDR",    00762,    00300, TYPE_E    },
    { "RDS",    00762,    0,     TYPE_E    },
+   { "REF",    REF_T,    0,     TYPE_P    },
    { "REM",    REM_T,    0,     TYPE_P    },
    { "RETURN", RETURN_T, 0,     TYPE_P    },
    { "REW",    00772,    0,     TYPE_B    },
@@ -298,6 +308,7 @@ OpCode optable[NUMOPS] =
    { "SCHH",   04643,    0,     TYPE_B    },
    { "SDN",    00776,    0,     TYPE_B    },
    { "SET",    SET_T,    0,     TYPE_P    },
+   { "SEVEN",  07000,    0,     TYPE_A    },
    { "SIB",    SIB_T,    0,     TYPE_P    },
    { "SIL",    04055,    0,     TYPE_D    },
    { "SIR",    00055,    0,     TYPE_D    },
@@ -368,6 +379,7 @@ OpCode optable[NUMOPS] =
    { "TEFF",   04032,    0,     TYPE_B    },
    { "TEFG",   00033,    0,     TYPE_B    },
    { "TEFH",   04033,    0,     TYPE_B    },
+   { "THREE",  03000,    0,     TYPE_A    },
    { "TIF",    00046,    0,     TYPE_B    },
    { "TIO",    00042,    0,     TYPE_B    },
    { "TITLE",  TITLE_T,  0,     TYPE_P    },
@@ -411,6 +423,7 @@ OpCode optable[NUMOPS] =
    { "VDP",    00225,    0,     TYPE_C    },
    { "VFD",    VFD_T,    0,     TYPE_P    },
    { "VLM",    00204,    0,     TYPE_C    },
+   { "WDR",    00766,    00300, TYPE_E    },
    { "WEF",    00770,    0,     TYPE_E    },
    { "WPB",    00766,    00362, TYPE_E    },
    { "WPD",    00766,    00361, TYPE_E    },
@@ -420,11 +433,14 @@ OpCode optable[NUMOPS] =
    { "WTB",    00766,    00220, TYPE_E    },
    { "WTD",    00766,    00200, TYPE_E    },
    { "WTR",    00000,    0,     TYPE_CHAN },
+   { "WTV",    00766,    00030, TYPE_E    },
    { "XCA",    00131,    0,     TYPE_B    },
-   { "XCL",    04131,    0,     TYPE_B    },
+   { "XCL",    04130,    0,     TYPE_B    },
    { "XEC",    00522,    0,     TYPE_B    },
+   { "XIT",    00021,    0,     TYPE_B    },
    { "XMT",    00000,    02,    TYPE_A    },
    { "ZAC",    04754,    0,     TYPE_B    },
+   { "ZERO",   00000,    0,     TYPE_A    },
    { "ZET",    00520,    0,     TYPE_B    },
    { "ZSA",    00634,    0,     TYPE_B    },
    { "ZSD",    04634,    0,     TYPE_B    },
@@ -435,11 +451,23 @@ OpCode optable[NUMOPS] =
 */
 
 extern int opdefcount;		/* Number of user defined opcode in opdef */
+extern OpDefCode *freeops;	/* Reusable opdef codes */
 extern OpDefCode *opdefcode[MAXDEFOPS];/* The user defined opcode table */
 
 static char retopcode[MAXSYMLEN+2]; /* Return opcode symbol buffer */
 static OpCode retop;		/* Return opcode buffer */
 
+/***********************************************************************
+* freeopd - Link a opcode entry on free chain.
+***********************************************************************/
+
+void
+freeopd (OpDefCode *or)
+{
+   or->next = freeops;
+   freeops = or;
+}
+
 /***********************************************************************
 * stdlookup - Lookup standard opcode. Uses binary search.
 ***********************************************************************/
@@ -591,7 +619,7 @@ oplookup (char *op)
 	 opbf[i] = '\0';
 	 if (i == 3 && opch >= 'A' && opch <= 'H')
 	 {
-	    if (ret = stdlookup (opbf))
+	    if ((ret = stdlookup (opbf)) != NULL)
 	    {
 	       strcpy (retopcode, op);
 	       retop.opcode = retopcode;
@@ -635,7 +663,12 @@ opadd (char *op, unsigned c0, unsigned c1, int type)
       exit (ABORT);
    }
 
-   if ((new = (OpDefCode *)malloc (sizeof (OpDefCode))) == NULL)
+   if (freeops)
+   {
+      new = freeops;
+      freeops = new->next;
+   }
+   else if ((new = (OpDefCode *)malloc (sizeof (OpDefCode))) == NULL)
    {
       fprintf (stderr, "asm7090: Unable to allocate memory\n");
       exit (ABORT);
